@@ -17,8 +17,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   see <http://www.gnu.org/licenses/>.
 
    Authors: Ettore Perazzoli <ettore@gnu.org>,
             Darin Adler <darin@bentspoon.com>,
@@ -55,4 +54,7 @@ void   nautilus_canvas_dnd_begin_drag            (NautilusCanvasContainer *conta
 						  int                    start_y);
 void   nautilus_canvas_dnd_end_drag              (NautilusCanvasContainer *container);
 
+NautilusDragInfo* nautilus_canvas_dnd_get_drag_source_data (NautilusCanvasContainer *container,
+                                                            GdkDragContext          *context);
+
 #endif /* NAUTILUS_CANVAS_DND_H */
