@@ -34,13 +34,14 @@ static const char *default_column_order[] = {
 	"name",
 	"size",
 	"type",
-	"date_modified",
-	"date_accessed",
 	"owner",
 	"group",
 	"permissions",
 	"mime_type",
 	"where",
+	"date_modified_with_time",
+	"date_modified",
+	"date_accessed",
 	NULL
 };
 
@@ -62,7 +63,6 @@ get_builtin_columns (void)
 					       "attribute", "size",
 					       "label", _("Size"),
 					       "description", _("The size of the file."),
-					       "xalign", 1.0,
 					       NULL));
 	columns = g_list_append (columns,
 				 g_object_new (NAUTILUS_TYPE_COLUMN,
@@ -78,6 +78,7 @@ get_builtin_columns (void)
 					       "label", _("Modified"),
 					       "description", _("The date the file was modified."),
 					       "default-sort-order", GTK_SORT_DESCENDING,
+					       "xalign", 1.0,
 					       NULL));
 	columns = g_list_append (columns,
 				 g_object_new (NAUTILUS_TYPE_COLUMN,
@@ -86,6 +87,7 @@ get_builtin_columns (void)
 					       "label", _("Accessed"),
 					       "description", _("The date the file was accessed."),
 					       "default-sort-order", GTK_SORT_DESCENDING,
+					       "xalign", 1.0,
 					       NULL));
 
 	columns = g_list_append (columns,
@@ -127,6 +129,14 @@ get_builtin_columns (void)
 					       "label", _("Location"),
 					       "description", _("The location of the file."),
 					       NULL));
+	columns = g_list_append (columns,
+				 g_object_new (NAUTILUS_TYPE_COLUMN,
+					       "name", "date_modified_with_time",
+					       "attribute", "date_modified_with_time",
+					       "label", _("Modified - Time"),
+					       "description", _("The date the file was modified."),
+					       "xalign", 1.0,
+					       NULL));
 
 	return columns;
 }
@@ -168,6 +178,7 @@ get_trash_columns (void)
 						       "attribute", "trashed_on",
 						       "label", _("Trashed On"),
 						       "description", _("Date when file was moved to the Trash"),
+					               "xalign", 1.0,
 						       NULL));
 		columns = g_list_append (columns,
 			                 g_object_new (NAUTILUS_TYPE_COLUMN,
