@@ -45,10 +45,6 @@ struct NautilusDesktopLinkMonitorDetails {
 	NautilusDesktopLink *trash_link;
 	NautilusDesktopLink *network_link;
 
-	gulong mount_id;
-	gulong unmount_id;
-	gulong changed_id;
-	
 	GList *mount_links;
 };
 
@@ -72,64 +68,6 @@ nautilus_desktop_link_monitor_get (void)
 	return the_link_monitor;
 }
 
-static void
-volume_delete_dialog (GtkWidget *parent_view,
-                      NautilusDesktopLink *link)
-{
-	GMount *mount;
-	char *dialog_str;
-	char *display_name;
-
-	mount = nautilus_desktop_link_get_mount (link);
-
-	if (mount != NULL) {
-		display_name = nautilus_desktop_link_get_display_name (link);
-		dialog_str = g_strdup_printf (_("You cannot move the volume “%s” to the trash."),
-					      display_name);
-		g_free (display_name);
-
-		if (g_mount_can_eject (mount)) {
-			eel_run_simple_dialog
-				(parent_view, 
-				 FALSE,
-				 GTK_MESSAGE_ERROR,
-				 dialog_str,
-				 _("If you want to eject the volume, please use Eject in the "
-				   "popup menu of the volume."),
-				 _("_OK"), NULL);
-		} else {
-			eel_run_simple_dialog
-				(parent_view, 
-				 FALSE,
-				 GTK_MESSAGE_ERROR,
-				 dialog_str,
-				 _("If you want to unmount the volume, please use Unmount Volume in the "
-				   "popup menu of the volume."),
-				 _("_OK"), NULL);
-		}
-
-		g_object_unref (mount);
-		g_free (dialog_str);
-	}
-}
-
-void
-nautilus_desktop_link_monitor_delete_link (NautilusDesktopLinkMonitor *monitor,
-					   NautilusDesktopLink *link,
-					   GtkWidget *parent_view)
-{
-	switch (nautilus_desktop_link_get_link_type (link)) {
-	case NAUTILUS_DESKTOP_LINK_HOME:
-	case NAUTILUS_DESKTOP_LINK_TRASH:
-	case NAUTILUS_DESKTOP_LINK_NETWORK:
-		/* just ignore. We don't allow you to delete these */
-		break;
-	default:
-		volume_delete_dialog (parent_view, link);
-		break;
-	}
-}
-
 static gboolean
 volume_file_name_used (NautilusDesktopLinkMonitor *monitor,
 		       const char *name)
@@ -416,16 +354,12 @@ nautilus_desktop_link_monitor_init (NautilusDesktopLinkMonitor *monitor)
 				  G_CALLBACK (desktop_volumes_visible_changed),
 				  monitor);
 
-	monitor->details->mount_id =
-		g_signal_connect_object (monitor->details->volume_monitor, "mount-added",
-					 G_CALLBACK (mount_added_callback), monitor, 0);
-	monitor->details->unmount_id =
-		g_signal_connect_object (monitor->details->volume_monitor, "mount-removed",
-					 G_CALLBACK (mount_removed_callback), monitor, 0);
-	monitor->details->changed_id =
-		g_signal_connect_object (monitor->details->volume_monitor, "mount-changed",
-					 G_CALLBACK (mount_changed_callback), monitor, 0);
-
+	g_signal_connect_object (monitor->details->volume_monitor, "mount-added",
+				 G_CALLBACK (mount_added_callback), monitor, 0);
+	g_signal_connect_object (monitor->details->volume_monitor, "mount-removed",
+				 G_CALLBACK (mount_removed_callback), monitor, 0);
+	g_signal_connect_object (monitor->details->volume_monitor, "mount-changed",
+				 G_CALLBACK (mount_changed_callback), monitor, 0);
 }
 
 static void
@@ -450,8 +384,6 @@ desktop_link_monitor_finalize (GObject *object)
 
 	monitor = NAUTILUS_DESKTOP_LINK_MONITOR (object);
 
-	g_object_unref (monitor->details->volume_monitor);
-
 	/* Default links */
 
 	remove_link_and_preference (&monitor->details->home_link,
@@ -482,15 +414,7 @@ desktop_link_monitor_finalize (GObject *object)
 					      desktop_volumes_visible_changed,
 					      monitor);
 
-	if (monitor->details->mount_id != 0) {
-		g_source_remove (monitor->details->mount_id);
-	}
-	if (monitor->details->unmount_id != 0) {
-		g_source_remove (monitor->details->unmount_id);
-	}
-	if (monitor->details->changed_id != 0) {
-		g_source_remove (monitor->details->changed_id);
-	}
+	g_object_unref (monitor->details->volume_monitor);
 
 	G_OBJECT_CLASS (nautilus_desktop_link_monitor_parent_class)->finalize (object);
 }
