@@ -14,8 +14,7 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; see the file COPYING.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * see <http://www.gnu.org/licenses/>.
  *
  * Author: Anders Carlsson <andersca@imendio.com>
  *
@@ -64,7 +63,5 @@ void           nautilus_query_add_mime_type      (NautilusQuery *query, const ch
 gdouble        nautilus_query_matches_string     (NautilusQuery *query, const gchar *string);
 
 char *         nautilus_query_to_readable_string (NautilusQuery *query);
-NautilusQuery *nautilus_query_load               (char *file);
-gboolean       nautilus_query_save               (NautilusQuery *query, char *file);
 
 #endif /* NAUTILUS_QUERY_H */
