@@ -347,7 +347,7 @@ static gboolean
 should_show_progress_info (NautilusProgressInfo *info)
 {
 
-        return nautilus_progress_info_get_elapsed_time (info) +
+        return nautilus_progress_info_get_total_elapsed_time (info) +
                nautilus_progress_info_get_remaining_time (info) > OPERATION_MINIMUM_TIME;
 }
 
@@ -379,7 +379,7 @@ should_hide_operations_button (NautilusToolbar *self)
         progress_infos = get_filtered_progress_infos (self);
 
         for (l = progress_infos; l != NULL; l = l->next) {
-                if (nautilus_progress_info_get_elapsed_time (l->data) +
+                if (nautilus_progress_info_get_total_elapsed_time (l->data) +
                     nautilus_progress_info_get_remaining_time (l->data) > OPERATION_MINIMUM_TIME &&
                     !nautilus_progress_info_get_is_cancelled (l->data) &&
                     !nautilus_progress_info_get_is_finished (l->data)) {
@@ -393,7 +393,7 @@ should_hide_operations_button (NautilusToolbar *self)
 }
 
 static gboolean
-remove_finished_operations (NautilusToolbar *self)
+on_remove_finished_operations_timeout (NautilusToolbar *self)
 {
         nautilus_progress_info_manager_remove_finished_or_cancelled_infos (self->priv->progress_manager);
         if (should_hide_operations_button (self)) {
@@ -423,12 +423,12 @@ schedule_remove_finished_operations (NautilusToolbar *self)
         if (self->priv->remove_finished_operations_timeout_id == 0) {
                 self->priv->remove_finished_operations_timeout_id =
                         g_timeout_add_seconds (REMOVE_FINISHED_OPERATIONS_TIEMOUT,
-                                               (GSourceFunc) remove_finished_operations,
+                                               (GSourceFunc) on_remove_finished_operations_timeout,
                                                self);
         }
 }
 
-static gboolean
+static void
 remove_operations_button_attention_style (NautilusToolbar *self)
 {
         GtkStyleContext *style_context;
@@ -436,11 +436,26 @@ remove_operations_button_attention_style (NautilusToolbar *self)
         style_context = gtk_widget_get_style_context (self->priv->operations_button);
         gtk_style_context_remove_class (style_context,
                                         "nautilus-operations-button-needs-attention");
+}
+
+static gboolean
+on_remove_operations_button_attention_style_timeout (NautilusToolbar *self)
+{
+        remove_operations_button_attention_style (self);
         self->priv->operations_button_attention_timeout_id = 0;
 
         return G_SOURCE_REMOVE;
 }
 
+static void
+unschedule_operations_button_attention_style (NautilusToolbar *self)
+{
+        if (self->priv->operations_button_attention_timeout_id!= 0) {
+                g_source_remove (self->priv->operations_button_attention_timeout_id);
+                self->priv->operations_button_attention_timeout_id = 0;
+        }
+}
+
 static void
 add_operations_button_attention_style (NautilusToolbar *self)
 {
@@ -448,13 +463,13 @@ add_operations_button_attention_style (NautilusToolbar *self)
 
         style_context = gtk_widget_get_style_context (self->priv->operations_button);
 
-        remove_operations_button_attention_style (self);
+        unschedule_operations_button_attention_style (self);
         remove_operations_button_attention_style (self);
 
         gtk_style_context_add_class (style_context,
                                      "nautilus-operations-button-needs-attention");
         self->priv->operations_button_attention_timeout_id = g_timeout_add (NEEDS_ATTENTION_ANIMATION_TIMEOUT,
-                                                                            (GSourceFunc) remove_operations_button_attention_style,
+                                                                            (GSourceFunc) on_remove_operations_button_attention_style_timeout,
                                                                             self);
 }
 
@@ -525,7 +540,7 @@ update_operations (NautilusToolbar *self)
         GList *progress_infos;
         GList *l;
         GtkWidget *progress;
-        guint total_remaining_time = 0;
+        gboolean should_show_progress_button = FALSE;
 
         gtk_container_foreach (GTK_CONTAINER (self->priv->operations_container),
                                (GtkCallback) gtk_widget_destroy,
@@ -535,7 +550,8 @@ update_operations (NautilusToolbar *self)
 
         progress_infos = get_filtered_progress_infos (self);
         for (l = progress_infos; l != NULL; l = l->next) {
-                total_remaining_time = nautilus_progress_info_get_remaining_time (l->data);
+                should_show_progress_button = should_show_progress_button ||
+                                              should_show_progress_info (l->data);
 
                 g_signal_connect_swapped (l->data, "finished",
 		                          G_CALLBACK (on_progress_info_finished), self);
@@ -560,7 +576,7 @@ update_operations (NautilusToolbar *self)
          * estimated time is longer than a OPERATION_MINIMUM_TIME is odd, so show
          * it only if the remaining time is bigger than again OPERATION_MINIMUM_TIME.
          */
-        if (total_remaining_time > OPERATION_MINIMUM_TIME &&
+        if (should_show_progress_button &&
             !gtk_revealer_get_reveal_child (GTK_REVEALER (self->priv->operations_revealer))) {
                 add_operations_button_attention_style (self);
                 gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->operations_revealer),
@@ -585,13 +601,13 @@ on_progress_info_started_timeout (NautilusToolbar *self)
         filtered_progress_infos = get_filtered_progress_infos (self);
         if (!nautilus_progress_manager_are_all_infos_finished_or_cancelled (self->priv->progress_manager) &&
             g_list_length (progress_infos) != g_list_length (filtered_progress_infos)) {
+                g_list_free (filtered_progress_infos);
                 return G_SOURCE_CONTINUE;
         } else {
+                g_list_free (filtered_progress_infos);
                 self->priv->start_operations_timeout_id = 0;
                 return G_SOURCE_REMOVE;
         }
-
-        g_list_free (filtered_progress_infos);
 }
 
 static void
@@ -789,7 +805,7 @@ nautilus_toolbar_set_property (GObject *object,
 }
 
 static void
-nautilus_toolbar_dispose (GObject *obj)
+nautilus_toolbar_finalize (GObject *obj)
 {
 	NautilusToolbar *self = NAUTILUS_TOOLBAR (obj);
 
@@ -799,16 +815,12 @@ nautilus_toolbar_dispose (GObject *obj)
 	unschedule_menu_popup_timeout (self);
         unschedule_remove_finished_operations (self);
         unschedule_operations_start (self);
-
-        if (self->priv->operations_button_attention_timeout_id != 0) {
-                g_source_remove (self->priv->operations_button_attention_timeout_id);
-                self->priv->operations_button_attention_timeout_id = 0;
-        }
+        unschedule_operations_button_attention_style (self);
 
         g_signal_handlers_disconnect_by_data (self->priv->progress_manager, self);
         g_clear_object (&self->priv->progress_manager);
 
-	G_OBJECT_CLASS (nautilus_toolbar_parent_class)->dispose (obj);
+	G_OBJECT_CLASS (nautilus_toolbar_parent_class)->finalize (obj);
 }
 
 static void
@@ -821,7 +833,7 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
 	oclass = G_OBJECT_CLASS (klass);
 	oclass->get_property = nautilus_toolbar_get_property;
 	oclass->set_property = nautilus_toolbar_set_property;
-	oclass->dispose = nautilus_toolbar_dispose;
+	oclass->finalize = nautilus_toolbar_finalize;
 
 	properties[PROP_WINDOW] =
 		g_param_spec_object ("window",
