@@ -490,6 +490,71 @@ on_slot_loading_changed (NautilusWindowSlot *slot,
                 on_location_changed (window);
 }
 
+static void
+notebook_switch_page_cb (GtkNotebook    *notebook,
+			 GtkWidget      *page,
+			 unsigned int    page_num,
+			 NautilusWindow *window)
+{
+	NautilusWindowSlot *slot;
+	GtkWidget *widget;
+
+	widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK (window->priv->notebook), page_num);
+	g_assert (widget != NULL);
+
+	/* find slot corresponding to the target page */
+	slot = NAUTILUS_WINDOW_SLOT (widget);
+	g_assert (slot != NULL);
+
+	nautilus_window_set_active_slot (nautilus_window_slot_get_window (slot),
+					 slot);
+}
+
+static void
+connect_slot (NautilusWindow     *window,
+              NautilusWindowSlot *slot)
+{
+        g_signal_connect (slot, "notify::loading",
+                          G_CALLBACK (on_slot_loading_changed), window);
+}
+
+static void
+disconnect_slot (NautilusWindow     *window,
+                 NautilusWindowSlot *slot)
+{
+        g_signal_handlers_disconnect_by_data (slot, window);
+}
+
+static NautilusWindowSlot *
+nautilus_window_create_slot (NautilusWindow             *window,
+			   NautilusWindowOpenFlags flags)
+{
+	NautilusWindowSlot *slot;
+
+	g_assert (NAUTILUS_IS_WINDOW (window));
+
+	slot = nautilus_window_slot_new (window);
+        connect_slot (window, slot);
+
+	g_signal_handlers_block_by_func (window->priv->notebook,
+					 G_CALLBACK (notebook_switch_page_cb),
+					 window);
+	nautilus_notebook_add_tab (NAUTILUS_NOTEBOOK (window->priv->notebook),
+				   slot,
+				   (flags & NAUTILUS_WINDOW_OPEN_SLOT_APPEND) != 0 ?
+				   -1 :
+				   gtk_notebook_get_current_page (GTK_NOTEBOOK (window->priv->notebook)) + 1,
+				   FALSE);
+	g_signal_handlers_unblock_by_func (window->priv->notebook,
+					   G_CALLBACK (notebook_switch_page_cb),
+					   window);
+
+	window->priv->slots = g_list_append (window->priv->slots, slot);
+	g_signal_emit (window, signals[SLOT_ADDED], 0, slot);
+
+	return slot;
+}
+
 void
 nautilus_window_open_location_full (NautilusWindow          *window,
                                     GFile                   *location,
@@ -510,7 +575,7 @@ nautilus_window_open_location_full (NautilusWindow          *window,
                 if (new_tab_at_end)
 		flags |= NAUTILUS_WINDOW_OPEN_SLOT_APPEND;
 
-		target_slot = nautilus_window_open_slot (window, flags);
+		target_slot = nautilus_window_create_slot (window, flags);
 	}
 
 	if (target_slot == NULL) {
@@ -620,41 +685,6 @@ location_entry_location_changed_callback (GtkWidget      *widget,
 	nautilus_window_open_location_full (window, location, 0, NULL, NULL);
 }
 
-static void
-notebook_switch_page_cb (GtkNotebook    *notebook,
-			 GtkWidget      *page,
-			 unsigned int    page_num,
-			 NautilusWindow *window)
-{
-	NautilusWindowSlot *slot;
-	GtkWidget *widget;
-
-	widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK (window->priv->notebook), page_num);
-	g_assert (widget != NULL);
-
-	/* find slot corresponding to the target page */
-	slot = NAUTILUS_WINDOW_SLOT (widget);
-	g_assert (slot != NULL);
-
-	nautilus_window_set_active_slot (nautilus_window_slot_get_window (slot),
-					 slot);
-}
-
-static void
-connect_slot (NautilusWindow     *window,
-              NautilusWindowSlot *slot)
-{
-        g_signal_connect (slot, "notify::loading",
-                          G_CALLBACK (on_slot_loading_changed), window);
-}
-
-static void
-disconnect_slot (NautilusWindow     *window,
-                 NautilusWindowSlot *slot)
-{
-        g_signal_handlers_disconnect_by_data (slot, window);
-}
-
 static void
 close_slot (NautilusWindow     *window,
 	    NautilusWindowSlot *slot,
@@ -684,36 +714,6 @@ close_slot (NautilusWindow     *window,
 	}
 }
 
-NautilusWindowSlot *
-nautilus_window_open_slot (NautilusWindow             *window,
-			   NautilusWindowOpenFlags flags)
-{
-	NautilusWindowSlot *slot;
-
-	g_assert (NAUTILUS_IS_WINDOW (window));
-
-	slot = nautilus_window_slot_new (window);
-        connect_slot (window, slot);
-
-	g_signal_handlers_block_by_func (window->priv->notebook,
-					 G_CALLBACK (notebook_switch_page_cb),
-					 window);
-	nautilus_notebook_add_tab (NAUTILUS_NOTEBOOK (window->priv->notebook),
-				   slot,
-				   (flags & NAUTILUS_WINDOW_OPEN_SLOT_APPEND) != 0 ?
-				   -1 :
-				   gtk_notebook_get_current_page (GTK_NOTEBOOK (window->priv->notebook)) + 1,
-				   FALSE);
-	g_signal_handlers_unblock_by_func (window->priv->notebook,
-					   G_CALLBACK (notebook_switch_page_cb),
-					   window);
-
-	window->priv->slots = g_list_append (window->priv->slots, slot);
-	g_signal_emit (window, signals[SLOT_ADDED], 0, slot);
-
-	return slot;
-}
-
 void
 nautilus_window_new_tab (NautilusWindow *window)
 {
@@ -977,6 +977,24 @@ build_selection_list_from_gfile_list (GList *gfile_list)
 	return g_list_reverse (result);
 }
 
+void
+nautilus_window_start_dnd (NautilusWindow *window,
+                           GdkDragContext *context)
+{
+        gtk_places_sidebar_set_drop_targets_visible (GTK_PLACES_SIDEBAR (window->priv->places_sidebar),
+                                                     TRUE,
+                                                     context);
+}
+
+void
+nautilus_window_end_dnd (NautilusWindow *window,
+                         GdkDragContext *context)
+{
+        gtk_places_sidebar_set_drop_targets_visible (GTK_PLACES_SIDEBAR (window->priv->places_sidebar),
+                                                     FALSE,
+                                                     context);
+}
+
 /* Callback used when the places sidebar needs to know the drag action to suggest */
 static GdkDragAction
 places_sidebar_drag_action_requested_cb (GtkPlacesSidebar *sidebar,
@@ -988,13 +1006,28 @@ places_sidebar_drag_action_requested_cb (GtkPlacesSidebar *sidebar,
 	GList *items;
 	char *uri;
 	int action = 0;
+        NautilusDragInfo *info;
+        guint32 source_actions;
 
-	items = build_selection_list_from_gfile_list (source_file_list);
+        info = nautilus_drag_get_source_data (context);
+        if (info != NULL) {
+                items = info->selection_cache;
+                source_actions = info->source_actions;
+        } else {
+                items = build_selection_list_from_gfile_list (source_file_list);
+                source_actions = 0;
+        }
 	uri = g_file_get_uri (dest_file);
 
-	nautilus_drag_default_drop_action_for_icons (context, uri, items, &action);
+        if (g_list_length (items) < 1)
+                goto out;
+
+	nautilus_drag_default_drop_action_for_icons (context, uri, items, source_actions, &action);
+
+out:
+        if (info == NULL)
+	        nautilus_drag_destroy_selection_list (items);
 
-	nautilus_drag_destroy_selection_list (items);
 	g_free (uri);
 
 	return action;
@@ -1997,7 +2030,7 @@ nautilus_window_constructed (GObject *self)
 	 * some actions trigger UI widgets to show/hide. */
 	nautilus_window_initialize_actions (window);
 
-	slot = nautilus_window_open_slot (window, 0);
+	slot = nautilus_window_create_slot (window, 0);
 	nautilus_window_set_active_slot (window, slot);
 
 	window->priv->bookmarks_id =
